/**
 * @file 	segmentationRLE.h
 * @brief	Header file for segmentationRLENew.c - segmentation with RLE 
 *		(Run Length Encoding) 
 *
 * @date  	Thu Nov 12 16:13:00 2003 - Original.
 * @author  Frederic Cupillard (Frederic.Cupillard@sophia.inria.fr).
 */


#ifndef _SEGMENTATION_RLE_H_
#define _SEGMENTATION_RLE_H_

#include <stdlib.h>

#include "ObserverTemplate.h"
#include "common.h"
#include "rectangle.h"

#include "RLEblob.h"
#include "RLEimage.h"
#include "RLEregion.h"

#include <opencv2/core/core.hpp>

namespace vsip {

  struct image;
  struct RLEblob;
  struct segment;
  struct region;
  struct scene_model;
  struct rectangle;
  struct RLEimage;

  /*! \def SEGM_FRAC_FRAME_NUMBER
   *  \brief number of sub-frames used for background updating. To save time, every time,
   *   we update only one part (sub-frame) of reference image. Usually, this value
   *   is number of frames taken per second. So after one second, whole image is 
   *   updated.
   */
#define SEGM_FRAC_FRAME_NUMBER		1 //3 /*5 *//*25*/

  /*! \def SEGM_SEGMENTS_MAX_NUMBER
   *  \brief number of ready segments in segment pool. Choose a big value for it to avoid
   * out of range error. So we don't need to check out of range error every time
   * (to save time)
   */
#define SEGM_SEGMENTS_MAX_NUMBER	100000


  /*! \def SEGM_REGIONS_MAX_NUMBER
   *  \brief number of ready regions in region pool. Choose a big value for it to avoid
   * out of range error. So we don't need to check out of range error every time
   * (to save time).
   */
#define SEGM_REGIONS_MAX_NUMBER		50000


  /*! \struct img2regs
   * \brief Struct contains data to build list connected regions from RLEimage.
   */
  struct img2regs {
    /*!
     * \var struct region* first_r
     * \brief pointer to the first region of region list
     */ 
    region_t first_r;

		 /*!
     * \var struct region* last_r
     * \brief pointer to the last region of region list . 
		*/
    region_t last_r;

		 /*!
     * \var int ycurrent
     * \brief current RLE line. 
		*/
    int ycurrent;

		 /*!
     * \var struct segment* curseg
     * \brief current segment on current RLE line . 
		*/
    segment_t curseg;

		 /*!
     * \var struct segment* precseg
     * \brief  current segment on preceding RLE line. 
		*/
    segment_t precseg;

		 /*!
     * \var struct segment* precl
     * \brief   point to preceding RLE line. 
		*/
    std::vector < segment_t >::iterator precl;

		 /*!
     * \var int nbprec
     * \brief  number of segments on preceding line. 
		*/
    int nbprec;

		 /*!
     * \var struct segment* curl
     * \brief  point to current RLE line. 
		*/
    std::vector < segment_t >::iterator curl;

		 /*!
     * \var int nbcurls
     * \brief  number of segments on current line. 
		*/
    int nbcurls;

		 /*!
     * \var int ixp
     * \brief  index of curseg segment. 
		*/
    int ixp;

		 /*!
     * \var int ixc
     * \brief  index of precseg segment. 
		*/
    int ixc;
  };

  /*! \typedef  typedef struct img2regs *img2regs_t 
	* \brief img2regs_t is a pointer to a img2regs structure. 
	*/
  typedef std::tr1::shared_ptr < struct img2regs > img2regs_t;

  //////////////////////////////////////////////////////////////////////////////
  //
  //   scene_model_seg
  //
  //////////////////////////////////////////////////////////////////////////////
  class MV_RLEsegm;

	/*! \enum SegmentType
	*  \brief segmentation classification.
	*/ 
  enum SegmentType {MOTION_YRATIO, /// CARETAKER : simple y channel ratio thresholding
    //For clear at each new frame history segmentation image
    INIT_BUILD_RLE_IMAGE,
    BACKGROUND,
		/*!< BACKGROUND
		*  pixel (x,y) is classified as Background
		*/
    SHADOW_YUV_HSV_TEXTURE,
		/*!< SHADOW_YUV_HSV_TEXTURE
    *  pixel (x,y) is classified as Shadow:
    * - YUV says this is a motion pixel,
    * - HSV says this is maybe a shadow,
    * - Texture analysis confirm this is a shadow.
		*/
    MOTION_YUV_HSV,
		/*!<  MOTION_YUV_HSV
    *  pixel (x,y) is classified as Motion:
    * - YUV says this is a motion pixel,
    * - HSV says this is not a shadow.
		*/
    MOTION_YUV_HSV_TEXTURE,
			/*!<  MOTION_YUV_HSV_TEXTURE
    *    pixel (x,y) is classified as Motion:
    * - YUV says this is a motion pixel,
    * - HSV says this is maybe a shadow,
    * - Texture analysis says this is not a shadow.
		*/
    MOTION_HSV,
		/*  MOTION_HSV
    *  pixel (x,y) is classified as Motion:
    * - HSV says this is not a shadow.
		*/
    BACKGROUND_POSSIBLY_SHADOW_HSV,
		/*!<  BACKGROUND_POSSIBLY_SHADOW_HSV
    *    pixel (x,y) is classified as Background
    * - HSV says this is maybe a shadow.
		*/
    MOTION_YUV,
		/* MOTION_YUV
    *  pixel (x,y) is classified as Motion:
    * - YUV says this is a motion pixel.
		*/
    MOTION_RGB,
		/* MOTION_RGB
    *  pixel (x,y) is classified as Motion:
    * - RGB says this is a motion pixel.
		*/
    MOTION_GREY,
		/*  MOTION_GREY
    *  pixel (x,y) is classified as Motion:
    * - GREY says this is a motion pixel.
		*/
    BACKGROUND_SUREXPOSITION_HSV,
			/* BACKGROUND_SUREXPOSITION_HSV
    *  pixel (x,y) is classified as Background:
    * - YUV says this is a motion pixel,
    * - HSV says this is a surexposition pixel.
		*/
    BACKGROUND_LOW_RGB_DIFF,
		/*!<  BACKGROUND_LOW_RGB_DIFF
    *   pixel (x,y) is classified as Background:
    * DR,DG and DB are very small so that it is considred as background
		*/
    MOTION_HSV_DS,
		/*!<  MOTION_HSV_DS
    *   pixel (x,y) is classified as Motion
		*  - HSV DSaturation says this is a motion pixel
		*/
    MOTION_HSV_DH,
			/*!<  MOTION_HSV_DH
    *    pixel (x,y) is classified as Motion
		*  - HSV DHue says this is a motion pixel
		*/
    MOTION_YUV_DUDV,
		/*!<  MOTION_YUV_DUDV
    *   pixel (x,y) is classified as Motion
		*  - YUV DU & DV  says this is a motion pixel
		*/
    MOTION_YUV_CLEAR_DY,
		/*!<  MOTION_YUV_CLEAR_DY
    *    pixel (x,y) is classified as Motion
		*  - YUV DY says this is a motion pixel
		*/
    MOTION_YUV_DARK_DY
		/*!<  MOTION_YUV_DARK_DY
    *    pixel (x,y) is classified as Motion
		*  - YUV DY says this is a motion pixel
		*/
  };

  typedef enum SegmentType SegmentType;

  class CSegmentMessage {
  public:
    CSegmentMessage(MV_RLEsegm& i_data) : m_data(i_data), m_x(0), m_y(0), m_gain(0.0) {}

    inline SegmentType  getType() const { return m_type; }
    inline void setType(SegmentType i_type){ m_type = i_type; }

    inline MV_RLEsegm& getData() const { return m_data; }
    inline int getX() const { return m_x; }
    inline int getY() const { return m_y; }
    inline double getGain() const { return m_gain; }
    inline void setX(int i_x) { m_x = i_x; }
    inline void setY(int i_y) { m_y = i_y; }
    inline void setGain(double i_gain) { m_gain = i_gain; }

  private:
    MV_RLEsegm&      m_data;
    SegmentType      m_type;
    int              m_x;
    int              m_y;
    double           m_gain;
  };

  /*! \enum SegmentMethod
	* \brief available segmentation methods : YUV_SHADOW_METHOD and ADAPT_GAIN_VARIATION_METHOD
	*/
  enum SegmentMethod {YRATIO_METHOD, /// CARETAKER : simple y channel ratio thresholding
    HSV_METHOD,
    YUV_METHOD,
    YUV_SHADOW_METHOD,
		/*!< YUV_SHADOW_METHOD
		* method CMovingPointYUVShadow
		*/
    ADAPT_GAIN_VARIATION_METHOD,
			/*!< YUV_SHADOW_METHOD
		* method CMovingPointAdaptGainVariation
		*/
    RGB_METHOD,
    GREY_METHOD
  };

  typedef enum SegmentMethod SegmentMethod;

  /*! \class MV_RLEsegm
   * \brief Main segmentation variables container. Contains pointer to current segmentation method (CMovingPoint)
   */

  class MV_RLEsegm : public Observable<CSegmentMessage>
  {
  public:
	 /*! \fn MV_RLEsegm()
	 * \brief MV_RLEsegm default constructor
	 */
	    MV_RLEsegm();

    ~MV_RLEsegm();

    /*! \var struct RLEimage   *mv_RLEsegm_rle_img
		* \brief  pointer to  RLE image 
		*/
    RLEimage_t mv_RLEsegm_rle_img;

    /*! \var struct img2regs   *mv_img2regs
		* \brief pointer to Structure for computing the connected regions from RLE image 
		*/
    img2regs_t  mv_img2regs;

    /*! \var struct segment    *mv_RLEsegm_seg_pool
     * \brief  Pointer on segment pool that contains all segments. Segment pool 
     * contains a great number of segments pre-allocated.
     */
    std::vector < segment_t > mv_RLEsegm_seg_pool;
    
    /*! \var int	            mv_RLEsegm_seg_index
		* \brief Index of last used segment 
		*/
    int	            mv_RLEsegm_seg_index;

    /*! \var struct region   *mv_RLEsegm_reg_pool
     * \brief  Pointer on region pool contains all regions in image. Region pool 
     * contains a great number of regions pre-allocated.
     */
    std::vector < region_t > mv_RLEsegm_reg_pool;

    /*! \var int  mv_RLEsegm_reg_index
		* \brief Index of last used region 
		*/
    int               mv_RLEsegm_reg_index;

    /*! \var int mv_RLEsegm_bg_update_coun
		* \brief ID for the subframe on background image that will be updated 
		*/
    int               mv_RLEsegm_bg_update_count;

    /*! \var struct blob	   *mv_blob_list[SEGM_BLOBS_MAX_NUMBER]
     * \brief  Array of SEGM_BLOBS_MAX_NUMBER blobs. Some blobs in this array will be
     * pre-allocated. SEGM_BLOBS_MAX_NUMBER is a value chosen for maximal 
     * number of possible detected blobs for a image.
     */
    //struct RLEblob	   *mv_blob_list[SEGM_BLOBS_MAX_NUMBER];
    std::vector < std::tr1::shared_ptr < struct RLEblob > >	mv_blob_list;

    /*! \var int  mv_RLEsegm_blobs_index
		* \brief Last index used for mv_blob_list table 
		*/
    int              mv_RLEsegm_blobs_index;
    /*! \var int  mv_RLEsegm_allocated_blobs
     * \brief  Number of blobs (in blob_list) already allocated. To avoid use too much
     * memory, first, only some blobs are allocated. Only when the number of
     * detected blobs is > number of pre-allocated blobs, other blobs will be
     * allocate. If number of blobs > SEGM_BLOBS_MAX_NUMBER, take account only
     * the first blobs.
     */
    int              mv_RLEsegm_allocated_blobs;
    /*! \var struct blob *mv_blob_list_head
		* \brief the first blob in list 
		*/
    blob_t mv_blob_list_head;
    /*! \var struct blob	   *mv_blob_list_tail
		* \brief the last blob in list 
		*/
    blob_t mv_blob_list_tail;

    /*! \var int mv_RLEsegm_blobs_number
		* \brief Number of detected blobs. 
		*/
    int              mv_RLEsegm_blobs_number;

    /*! \var int  mv_RLEsegm_xleft
     * \brief  Region of interest (ROI) xleft coordinate the segmentation is performed in.
     */
    int              mv_RLEsegm_xleft;
		 /*! \var int  mv_RLEsegm_ytop
     * \brief  Region of interest (ROI) ytop coordinate the segmentation is performed in.
     */
    int              mv_RLEsegm_ytop;
		 /*! \var int  mv_RLEsegm_xright
     * \brief  Region of interest (ROI) xright coordinate the segmentation is performed in.
     */
    int              mv_RLEsegm_xright;
		 /*! \var int  mv_RLEsegm_ybottom
     * \brief  Region of interest (ROI) ybottom coordinate the segmentation is performed in.
     */
    int              mv_RLEsegm_ybottom;

    /*! \var struct rectangle*      mv_RLEsegm_updating_bbox
     * \brief Pool of sub-frames using for update background. To save time, for every 
     * instant, not update whole reference image but one sub-zone. Number of
     * sub-zone defined by SEGM_FRAC_FRAME_NUMBER. Usually, this value is number 
     * of images taken per second. So after one second, whole image is updated.
     * This variable points to the first sub-zone in array of sub-zones.
     */
    //struct rectangle*      mv_RLEsegm_updating_bbox;

		/*! \var int _RLEsegm_xstep
		* \brief the x-step between 2 consecutives pixels the segmentation method is applied on
		*/
    int _RLEsegm_xstep;
		/*! \var int _RLEsegm_ystep
		* \brief the y-step between 2 consecutives pixels the segmentation method is applied on
		*/
    int _RLEsegm_ystep;

    //double _RLEsegm_a_bg;
    //double _RLEsegm_a_fg;

    int threshold1;
    int threshold2;

		/*! \var int min_dimension
		* \brief min Height and Width a blob must have at least to be created a the end of the segmentation process
		*/
    int min_dimension;
		/*! \var int min_area
		* \brief min dimension (Height*Width) a blob must have at least to be created a the end of the segmentation process
		*/
    int min_area;

    int NbrDetectedBlobsCur;
    int DimDetectedBlobsCur;
    int NbrDetectedBlobsPrev;
    int DimDetectedBlobsPrev;

    std::map<rectangle_t, double> m_RectScore;

    bool m_isFrameDiff;

    /*! \fn struct RLEimage* RLEsegm_build_RLEimage(struct image* cur_img, struct scene_model* pTp_sm_node, bool is_intensityVariationcomputed)
    * \brief Build a RLE image from current image and reference image. 
    * \param cur_img pointer to the current image
		* \param pTp_sm_node pointer to the scene model 
		* \param is_intensityVariationcomputed tell if the intensity variation between current image and updated reference image is available 
    * \return pointer to RLE struct image.
    */
    RLEimage_t RLEsegm_build_RLEimageSUP ( const cv::Mat& );


    /*! \fn void RLEsegm_build_RLEline(int l, struct scene_model* pTp_sm_node)
     * \brief Build a RLEline of a RLEimage.
     * \param l Line offset in RLEimage.
		 * \param pTp_sm_node pointer to the scene model 
     */
    void RLEsegm_build_RLElineSUP(int, const cv::Mat& );

    bool RLEsegm_is_moving_point_in_mask(int x, int y);
    bool RLEsegm_is_moving_point_in_mask(int x, int y, std::map<rectangle_t, double> *pRectScore, rectangle_t pRect);
                                         
    //std::map<rectangle_t, double> *GetScoreRectMap() {return m_pRectScore;}                                          
    //void setScoreRectMap(std::map<rectangle_t, double> *pRectScore) { m_pRectScore = pRectScore;}

    //Modified by Marcos Zuniga
    //void RLEsegm_updateMainParameters(double *parameters);

    inline CSegmentMessage& getMsg() { return m_message; }
    inline SegmentType getMsgType() { return m_message.getType(); }
    inline void setMsgType(SegmentType i_type) { m_message.setType(i_type); }
    inline int getMsgX() const { return m_message.getX(); }
    inline int getMsgY() const { return m_message.getY(); }
    inline double getMsgGain() const { return m_message.getGain(); }
    inline void setMsgX(int i_x) { m_message.setX(i_x); }
    inline void setMsgY(int i_y) { m_message.setY(i_y); }
    inline void setMsgGain(double i_Gain) { m_message.setGain(i_Gain); }
    //inline struct image* getMaskImage() { return mv_RLEsegm_mask_image;}
    //inline void setMaskImage(struct image* i_maskImage) { mv_RLEsegm_mask_image = i_maskImage; }

  private:

    /*! \var struct image* mv_RLEsegm_mask_image
     * \brief Mask Image. No segmentation is done for current pixel with coordinate in black mask image.
     */
    //struct image* mv_RLEsegm_mask_image;

    CSegmentMessage  m_message;
  };

  typedef std::tr1::shared_ptr < MV_RLEsegm > MV_RLEsegm_t;


//#define MV_IMG_CUR(segm)        ((segm)->mv_RLEsegm_cur_img)
//#define MV_IMG_BG_REF(segm)     ((segm)->mv_RLEsegm_ref_img)
//#define MV_IMG_BG_FRAC(segm)    ((segm)->mv_RLEsegm_frac_img)
//#define MV_IMG_BIN(segm)        ((segm)->mv_RLEsegm_bin_img)
#define MV_IMG_RLE(segm)        ((segm)->mv_RLEsegm_rle_img)
#define MV_BG_UPD_COUNT(segm)   ((segm)->mv_RLEsegm_bg_update_count)
#define MV_SEGS_POOL(segm)      ((segm)->mv_RLEsegm_seg_pool)
#define MV_SEG_INDEX(segm)      ((segm)->mv_RLEsegm_seg_index)
#define MV_REGS_POOL(segm)      ((segm)->mv_RLEsegm_reg_pool)
#define MV_REG_INDEX(segm)      ((segm)->mv_RLEsegm_reg_index)
#define MV_IMG2REGS(segm)       ((segm)->mv_img2regs)
#define MV_BLOB_LIST(segm)      ((segm)->mv_blob_list)
#define MV_BLOBS_LIST_IX(segm)    ((segm)->mv_RLEsegm_blobs_index)
#define MV_BLOBS_LIST_HEAD(segm)  ((segm)->mv_blob_list_head)
#define MV_BLOBS_LIST_TAIL(segm)  ((segm)->mv_blob_list_tail)
#define MV_BLOBS_NUM(segm)        ((segm)->mv_RLEsegm_blobs_number)
#define MV_ALLOCATED_BLOBS(segm)  ((segm)->mv_RLEsegm_allocated_blobs)
#define MV_RLESEGM_XLEFT(segm)    ((segm)->mv_RLEsegm_xleft)
#define MV_RLESEGM_YTOP(segm)     ((segm)->mv_RLEsegm_ytop)
#define MV_RLESEGM_XRIGHT(segm)   ((segm)->mv_RLEsegm_xright)
#define MV_RLESEGM_YBOTTOM(segm)  ((segm)->mv_RLEsegm_ybottom)
//#define MV_GAIN(segm)             ((segm)->mv_gain)
//#define MV_IS_INTENSITY_VAR_AVAILABLE(segm) ((segm)->mv_isIntensityVariationAvailable)
//#define MV_ADAPT_CHANGES(segm)    ((segm)->mv_adaptChangesAdress)
#define MV_RLESEGM_UPDATING_BBOX(segm)   ((segm)->mv_RLEsegm_updating_bbox)
#define MV_RLESEGM_MOVING_FN(segm)       ((segm)->m_moving_point)
#define MV_RLESEGM_XSTEP(segm)           ((segm)->_RLEsegm_xstep)
#define MV_RLESEGM_YSTEP(segm)           ((segm)->_RLEsegm_ystep)
#define MV_RLESEGM_INTEG_BG(segm)        ((segm)->_RLEsegm_a_bg)
#define MV_RLESEGM_INTER_FG(segm)        ((segm)->_RLEsegm_a_fg)
#define MV_RLESEGM_THRESHOLD1(segm)      ((segm)->threshold1)
#define MV_RLESEGM_THRESHOLD2(segm)      ((segm)->threshold2)
#define MV_RLESEGM_MIN_DIMENSION(segm)  ((segm)->min_dimension)
#define MV_RLESEGM_MIN_AREA(segm)        ((segm)->min_area)
#define MV_NB_DETECTED_BLOBS_CUR(segm)        ((segm)->NbrDetectedBlobsCur)
#define MV_DIM_DETECTED_BLOBS_CUR(segm)        ((segm)->DimDetectedBlobsCur)
#define MV_NB_DETECTED_BLOBS_PREV(segm)        ((segm)->NbrDetectedBlobsPrev)
#define MV_DIM_DETECTED_BLOBS_PREV(segm)        ((segm)->DimDetectedBlobsPrev)
	/**
	 * Pre_allocated for segment pool.
	 * @param size Number of segments to allocate.
	 */
	std::vector < segment_t >  RLEsegm_seg_pool_init(int size, MV_RLEsegm_t pCurrentSegmentation);


	/**
 	 * Pre_allocated for region pool.
 	 * @param size Number of regions to allocate.
 	 */
	std::vector < region_t > RLEsegm_reg_pool_init(int size, MV_RLEsegm_t pCurrentSegmentation);

  /**
   * Get a region from its id.
   * @param reg_id id of wanted region.
   * @return wanted region if OK, NULL if reg_id pass beyond number of regions.
   */
  region_t RLEsegm_get_region(int reg_id, MV_RLEsegm_t pCurrentSegmentation);

  void RLEsegm_set_steps(MV_RLEsegm_t pCurrentSegmentation, const int xstep, const int ystep);


  /**
   * save binaire image (result of segmentation) to PPM file.
   * @param filename target file.
   * @param blobs list of blobs detected, we used them only to draw a bouding box
   * for every blob detected on binaire image.
   * @note save to  PPM format.
   */
  void RLEsegm_save_binaire(char   *filepath, struct RLEblob* blobs, MV_RLEsegm_t pCurrentSegmentation);


  /** Create new region from a segment */
  void img2regs_new_region(img2regs_t 	i2r, int x0, struct segment* seg, MV_RLEsegm_t pCurrentSegmentation);


  /** initialize a img2regs structure */
  void img2regs_init(img2regs_t i2r, MV_RLEsegm_t pCurrentSegmentation);

  /**
   *advance current segment on current RLE line , phase of connecting regions.
   */
  void img2regs_avanc(img2regs_t i2r, MV_RLEsegm_t pCurrentSegmentation);

  /**
   * Advance current segment on preceding RLE line , phase of connecting regions.
   */
  void img2regs_avanp(img2regs_t i2r);

  /**
   * Update a img2regs structure from current RLE line, phase of connecting
   * regions.
   */
  void img2regs_initstep(img2regs_t i2r,MV_RLEsegm_t pCurrentSegmentation);

  /**
   * Replace a region by another, phase of connecting regions.
   */
  void img2regs_replace(img2regs_t i2r, region_t r2, region_t r1);


  /**
   * Fusion two regions, phase of connecting regions.
   */
  void img2regs_fusion(img2regs_t i2r);

  /**
   * Starting test of segment connecting betwen current RLE line and preceding
   * RLE line, phase of connecting regions.
   */
  void img2regs_iter(img2regs_t i2r, MV_RLEsegm_t pCurrentSegmentation);


  /**
   * Starting test of segment connecting for all RLE line, phase of connecting
   * regions.
   */
  void img2regs_exec(img2regs_t 	i2r,
                     int 		size, MV_RLEsegm_t pCurrentSegmentation);


  /** Updating the regions after phase of merging */
  void img2regs_pass2(img2regs_t i2r, MV_RLEsegm_t pCurrentSegmentation);

  /**
   * get next available blob from blob pool.
   * @return address of new blob.
   */
  blob_t RLEsegm_get_new_blob(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * append new blob to list of blobs (defined in segmentationRLE.c).
   * @param new_blob new blob to append.
   */
  void RLEsegm_blob_list_append(struct RLEblob* new_blob, MV_RLEsegm_t pCurrentSegmentation);

  void RLEsegm_blob_list_append_to_head(struct RLEblob* new_blob, MV_RLEsegm_t pCurrentSegmentation);

  /**
   * remove a blob from list of blobs (defined in segmentationRLE.c).
   * @param blob blob to be removed.
   */
  void RLEsegm_blob_list_remove_blob(struct RLEblob* blob, MV_RLEsegm_t pCurrentSegmentation);

  /** Set ROI for segmentation */
  void RLEsegm_set_ROI(int xi,
                  int yi,
                  int xf,
                  int yf,
                  MV_RLEsegm_t pCurrentSegmentation  );


  /** Pre-compute size of sub-zones using for updating reference image. */
  void RLEsegm_build_updating_bbox(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * initialize the private segmentation variables (for each context).
   * @param ref reference image.
   * @note this function will be called ones for each context.
   */
  void RLEsegm_init_privateSUP ( MV_RLEsegm_t pCurrentSegmentation, int xleftROI,int xrightROI, int ytopROI, int ybotROI, int image_width, int image_height);
	
  void createAlgorithm(char i_colorSpace[],MV_RLEsegm_t o_pCurrentSegmentation);
	
  /**
   * initialize the two thresholds used in image segmentation. These
   * thresholds are stored in multi context node, because they may be
   * different for each camera.
   * Thresholds default values are stored in parameter file, but they
   * are adjusted taking into account the colour dynamic range of the image.
   * @note this function is called ones for each context.
   */
  void RLEsegm_init_thresholds(MV_RLEsegm_t pCurrentSegmentation, struct scene_model* pTp_sm_node, int threshold);

  /**
   * free the common segmentation variables.
   * @note This function will be called one time at the end of program.
   */
  void RLEsegm_free(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * free the private segmentation variables (for each context)
   * @note This function will be called one time for each context.
   */
  void RLEsegm_free_private(MV_RLEsegm_t pCurrentSegmentation);

  /** Reset RLE parameters for next frame */
  void RLEsegm_reset(MV_RLEsegm_t pCurrentSegmentation);


  /** Build list from the regions */
  blob_t RLEsegm_regions2blobsSUP ( int min_dimension, int min_num_pix, MV_RLEsegm_t pCurrentSegmentation );


  /**
   * get number of blobs detected.
   * @return number of blobs detected.
   */
  int RLEsegm_get_blobs_number(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * increase number of blobs 
   * @return new number of blobs detected.
   */
  int RLEsegm_increase_blobs_number(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * decrease number of blobs 
   * @return new number of blobs detected.
   */
  int RLEsegm_decrease_blobs_number(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * segment an image, detect the moving regions on image.
   * @note the parameters used for this function found at SEGEMENTATION
   * PARAMETERS section in parameter file (.dat).
   * @param cur_img image to segment.
   */
  blob_t RLEsegm_segmentSUP ( const cv::Mat& i_segmentedImage, MV_RLEsegm_t pCurrentSegmentation );

  void RLEreset_blob_idents(MV_RLEsegm_t pCurrentSegmentation);

  /**
   * Get sum of y coordinate of all pixels in a sub-region of a region.
   * @param reg_id region identificator.
   * @param x1 left coordinate of sub-region.
   * @param y1 top coordinate of sub-region.
   * @param x2 right coordinate of sub-region.
   * @param y2 bottom coordinate of sub-region.
   * @return sum of y coordinate of all pixels in sub-region.
   */
  double
  subregion_get_sumy(int reg_id,
                     int x1,
                     int y1,
                     int x2,
                     int y2,
                     MV_RLEsegm_t pCurrentSegm);



  /**
  * Get sum of y and x coordinate of all pixels in a sub-region of a region.
  * @param reg_id region identificator.
  * @param x1 left coordinate of sub-region.
  * @param y1 top coordinate of sub-region.
  * @param x2 right coordinate of sub-region.
  * @param y2 bottom coordinate of sub-region.
  * @param sumx pointer to sum of x coordinates of pixels in sub-region (output).
  * @param sumy pointer to sum of y coordinates of pixels in sub-region (output). 
  * @return number of pixels in sub-region.
  */
  int
  subregion_get_sumxsumy(int reg_id,
                         int x1,
                         int y1,
                         int x2,
                         int y2,
                         double *sumx,
                         double *sumy,
                         MV_RLEsegm_t pCurrentSegm);

  /* int subregion_get_sumxsumy(int reg_id,  */
  /* 			   int x1,  */
  /* 			   int y1,  */
  /* 			   int x2,  */
  /* 			   int y2, */
  /* 			   double *sumx,  */
  /* 			   double *sumy, */
  /* 			   MV_RLEsegm_t pRLEsegm); */

  /**
   * Get sum of x coordinate of all pixels in a sub-region of a region.
   * @param reg_id region identificator.
   * @param x1 left coordinate of sub-region.
   * @param y1 top coordinate of sub-region.
   * @param x2 right coordinate of sub-region.
   * @param y2 bottom coordinate of sub-region.
   * @return sum of x coordinate of all pixels in sub-region.
   */
  double
  subregion_get_sumx(int reg_id,
                     int x1,
                     int y1,
                     int x2,
                     int y2,
                     MV_RLEsegm_t pCurrentSegm);

  /**
   * Get limit value of y coordinate (y top and y bottom) for a sub-region of a 
   * region. Y top of sub-region is the y coordinate of the first moving point 
   * found when going through top-down. Y bottom of sub-region is the y 
   * coordinate of the first moving point found when going through bottom-up.
   * @param reg_id region identificator.
   * @param x1 left coordinate of sub-region.
   * @param y1 top coordinate of sub-region.
   * @param x2 right coordinate of sub-region.
   * @param y2 bottom coordinate of sub-region.
   * @param ytop pointer to y top of sub-region (output).
   * @param ybottom pointer to y bottom of sub-region (output).
   */
  int
  subregion_get_ylimit(int reg_id,
                       int x1,
                       int y1,
                       int x2,
                       int y2,
                       int *ytop,
                       int *ybottom,
                       MV_RLEsegm_t pCurrentSegm);

  /**
   * Get limit value of x coordinate (x left and x right) for a sub-region of a 
   * region. X left of sub-region is the x coordinate of the first moving point 
   * found when going through left->rightdown. X right of sub-region is the x 
   * coordinate of the first moving point found when going through right->left.
   * @param reg_id region identificator.
   * @param x1 left coordinate of sub-region.
   * @param y1 top coordinate of sub-region.
   * @param x2 right coordinate of sub-region.
   * @param y2 bottom coordinate of sub-region.
   * @param xleft pointer to y left of sub-region (output).
   * @param xright pointer to x right of sub-region (output).
   */
  int
  subregion_get_xlimit(int reg_id,
                       int x1,
                       int y1,
                       int x2,
                       int y2,
                       int *xleft,
                       int *xright,
                       MV_RLEsegm_t pCurrentSegm);
  /**
   * 	Get the index of the updated BBox in the range {0,1,2,...,SEGM_FRAC_FRAME_NUMBER-1}	
   *  @param x x-coordinate of pixel
   *  @param y y-coordinate of pixel
   */
  int RLEsegm_which_updating_bbox_pixel_belong_to(int x, int y, MV_RLEsegm_t pCurrentSegmentation);

  /**
   * 	Set list of region lines, which store segment indexes for region, per height	
   *  @param pCurrentSegmentation
   *  @param pTp_sm_node
   */
  void set_segs_in_regsSUP ( MV_RLEsegm_t pCurrentSegmentation, int i_height );


} // end namespace vsip

#endif /* _SEGMENTATION_RLE_H_ */


